#ifndef SCENETREE_H
#define SCENETREE_H

#include <vector>
#include <string>
#include "scene.h"
#include "light.h"

class SceneNode
{
public:
    SceneNode* m_parent;
    virtual void update() = 0;
};


class ContainerNode : public SceneNode
{
public:
    virtual void update();
    void addChild(const std::string &childName);
    void removeChild(const std::string &childName);
protected:
    std::vector<SceneNode*> m_children;
};

class SceneTree : public Scene
{
public:
    SceneTree(std::string name);
    ~SceneTree();

    virtual SceneIterator<Light*>* getLights();
    virtual SceneIterator<GeometryNode*>* getGeometry();

    void update();

    Texture* getSkybox() {return m_skybox;}
    void setSkybox(Texture* skybox) {m_skybox = skybox;}

private:
    std::string m_name;
    ContainerNode m_objects;
    ContainerNode m_map;
    Texture* m_skybox;
};

//general Node

class ObjectNode : public SceneNode
{
    //add bullet rigidbody
public:
    virtual void update();
};

class FragmentNode: public ObjectNode
{
    // TODO:  later
};

class ItemNode: public ObjectNode
{
    // TODO:  later
};

// Node
class GUINode : public SceneNode//, public GUIEntity
{
public:
    virtual void update();
};

class PlayerNode: public SceneNode//, public FPSCamera
{
public:
    virtual void update();
};

class EnvironmentNode : public SceneNode
{
public:
    virtual void update();
    // parametre skybox
    // directionnal light
    // parametre fog
};

class LightNode : public SceneNode, public Light
{
public:
    virtual void update();
    //position
    //puissance
    //lighting angle
    //couleur
};

class ChunkNode: public SceneNode
{
public:
    virtual void update();
//    map
};

#endif // SCENETREE_H
